@@ -17,8 +17,6 @@
 Command-line interface to the OpenStack Cinder API.
 """
 
-from __future__ import print_function
-
 import argparse
 import collections
 import getpass
@@ -33,32 +31,57 @@ from keystoneauth1 import loading
 from keystoneauth1 import session
 from oslo_utils import encodeutils
 from oslo_utils import importutils
-osprofiler_profiler = importutils.try_import("osprofiler.profiler")  # noqa
 import requests
 import six
 import six.moves.urllib.parse as urlparse
 
 import cinderclient
-from cinderclient import _i18n
 from cinderclient._i18n import _
 from cinderclient import api_versions
 from cinderclient import client
 from cinderclient import exceptions as exc
 from cinderclient import utils
 
-
-# Enable i18n lazy translation
-_i18n.enable_lazy()
+try:
+    osprofiler_profiler = importutils.try_import("osprofiler.profiler")
+except Exception:
+    pass
 
 
 DEFAULT_MAJOR_OS_VOLUME_API_VERSION = "3"
 DEFAULT_CINDER_ENDPOINT_TYPE = 'publicURL'
-V1_SHELL = 'cinderclient.v1.shell'
 V2_SHELL = 'cinderclient.v2.shell'
 V3_SHELL = 'cinderclient.v3.shell'
 HINT_HELP_MSG = (" [hint: use '--os-volume-api-version' flag to show help "
                  "message for proper version]")
 
+FILTER_CHECK = ["type-list",
+                "backup-list",
+                "get-pools",
+                "list",
+                "group-list",
+                "group-snapshot-list",
+                "message-list",
+                "snapshot-list",
+                "attachment-list"]
+
+RESOURCE_FILTERS = {
+    "list": ["name", "status", "metadata",
+             "bootable", "migration_status", "availability_zone",
+             "group_id", "size"],
+    "backup-list": ["name", "status", "volume_id"],
+    "snapshot-list": ["name", "status", "volume_id", "metadata",
+                      "availability_zone"],
+    "group-list": ["name"],
+    "group-snapshot-list": ["name", "status", "group_id"],
+    "attachment-list": ["volume_id", "status", "instance_id", "attach_status"],
+    "message-list": ["resource_uuid", "resource_type", "event_id",
+                     "request_id", "message_level"],
+    "get-pools": ["name", "volume_type"],
+    "type-list": ["is_public"]
+}
+
+
 logging.basicConfig()
 logger = logging.getLogger(__name__)
 
@@ -113,6 +136,7 @@ class OpenStackCinderShell(object):
     def __init__(self):
         self.ks_logger = None
         self.client_logger = None
+        self.extensions = []
 
     def get_base_parser(self):
         parser = CinderClientArgumentParser(
@@ -260,24 +284,14 @@ class OpenStackCinderShell(object):
         parser.add_argument('--os_password',
                             help=argparse.SUPPRESS)
 
-        # tenant_name is deprecated by project_name in keystoneauth
         parser.set_defaults(os_project_name=utils.env('OS_PROJECT_NAME',
-                                                      'OS_TENANT_NAME',
                                                       'CINDER_PROJECT_ID'))
-        parser.add_argument('--os_tenant_name',
-                            dest='os_project_name',
-                            help=argparse.SUPPRESS)
         parser.add_argument(
             '--os_project_name',
             help=argparse.SUPPRESS)
 
-        # tenant_id is deprecated by project_id in keystoneauth
         parser.set_defaults(os_project_id=utils.env('OS_PROJECT_ID',
-                                                    'OS_TENANT_ID',
-                                                    'CINDER_TENANT_ID'))
-        parser.add_argument('--os_tenant_id',
-                            dest='os_project_id',
-                            help=argparse.SUPPRESS)
+                                                    'CINDER_PROJECT_ID'))
         parser.add_argument(
             '--os_project_id',
             help=argparse.SUPPRESS)
@@ -341,12 +355,10 @@ class OpenStackCinderShell(object):
         self.subcommands = {}
         subparsers = parser.add_subparsers(metavar='<subcommand>')
 
-        if version.ver_major == 2:
-            actions_module = importutils.import_module(V2_SHELL)
-        elif version.ver_major == 3:
+        if version.ver_major == 3:
             actions_module = importutils.import_module(V3_SHELL)
         else:
-            actions_module = importutils.import_module(V1_SHELL)
+            actions_module = importutils.import_module(V2_SHELL)
 
         self._find_actions(subparsers, actions_module, version, do_help,
                            input_args)
@@ -521,8 +533,43 @@ class OpenStackCinderShell(object):
         else:
             return argv
 
+    @staticmethod
+    def _validate_input_api_version(options):
+        if not options.os_volume_api_version:
+            api_version = api_versions.APIVersion(api_versions.MAX_VERSION)
+        else:
+            api_version = api_versions.get_api_version(
+                options.os_volume_api_version)
+        return api_version
+
+    @staticmethod
+    def downgrade_warning(requested, discovered):
+        logger.warning("API version %s requested, " % requested.get_string())
+        logger.warning("downgrading to %s based on server support." %
+                       discovered.get_string())
+
+    def check_duplicate_filters(self, argv, filter):
+        resource = RESOURCE_FILTERS[filter]
+        filters = []
+        for opt in range(len(argv)):
+            if argv[opt].startswith('--'):
+                if argv[opt] == '--filters':
+                    key, __ = argv[opt + 1].split('=')
+                    if key in resource:
+                        filters.append(key)
+                elif argv[opt][2:] in resource:
+                    filters.append(argv[opt][2:])
+
+        if len(set(filters)) != len(filters):
+            raise exc.CommandError(
+                "Filters are only allowed to be passed once.")
+
     def main(self, argv):
         # Parse args once to find version and debug settings
+        for filter in FILTER_CHECK:
+            if filter in argv:
+                self.check_duplicate_filters(argv, filter)
+                break
         parser = self.get_base_parser()
         (options, args) = parser.parse_known_args(argv)
         self.setup_debugging(options.debug)
@@ -532,12 +579,7 @@ class OpenStackCinderShell(object):
         do_help = ('help' in argv) or (
             '--help' in argv) or ('-h' in argv) or not argv
 
-        if not options.os_volume_api_version:
-            api_version = api_versions.get_api_version(
-                DEFAULT_MAJOR_OS_VOLUME_API_VERSION)
-        else:
-            api_version = api_versions.get_api_version(
-                options.os_volume_api_version)
+        api_version = self._validate_input_api_version(options)
 
         # build available subcommands based on version
         major_version_string = "%s" % api_version.ver_major
@@ -630,11 +672,8 @@ class OpenStackCinderShell(object):
 
             if not project_info_provided:
                 raise exc.CommandError(_(
-                    "You must provide a tenant_name, tenant_id, "
-                    "project_id or project_name (with "
+                    "You must provide a project_id or project_name (with "
                     "project_domain_name or project_domain_id) via "
-                    "  --os-tenant-name (env[OS_TENANT_NAME]),"
-                    "  --os-tenant-id (env[OS_TENANT_ID]),"
                     "  --os-project-id (env[OS_PROJECT_ID])"
                     "  --os-project-name (env[OS_PROJECT_NAME]),"
                     "  --os-project-domain-id "
@@ -650,11 +689,8 @@ class OpenStackCinderShell(object):
 
         if not project_info_provided:
             raise exc.CommandError(_(
-                "You must provide a tenant_name, tenant_id, "
-                "project_id or project_name (with "
+                "You must provide a project_id or project_name (with "
                 "project_domain_name or project_domain_id) via "
-                "  --os-tenant-name (env[OS_TENANT_NAME]),"
-                "  --os-tenant-id (env[OS_TENANT_ID]),"
                 "  --os-project-id (env[OS_PROJECT_ID])"
                 "  --os-project-name (env[OS_PROJECT_NAME]),"
                 "  --os-project-domain-id "
@@ -673,9 +709,7 @@ class OpenStackCinderShell(object):
 
         insecure = self.options.insecure
 
-        self.cs = client.Client(
-            api_version, os_username,
-            os_password, os_project_name, os_auth_url,
+        client_args = dict(
             region_name=os_region_name,
             tenant_id=os_project_id,
             endpoint_type=endpoint_type,
@@ -683,7 +717,7 @@ class OpenStackCinderShell(object):
             service_type=service_type,
             service_name=service_name,
             volume_service_name=volume_service_name,
-            bypass_url=os_endpoint,
+            os_endpoint=os_endpoint,
             retries=options.retries,
             http_log_debug=args.debug,
             insecure=insecure,
@@ -692,6 +726,11 @@ class OpenStackCinderShell(object):
             session=auth_session,
             logger=self.ks_logger if auth_session else self.client_logger)
 
+        self.cs = client.Client(
+            api_version, os_username,
+            os_password, os_project_name, os_auth_url,
+            **client_args)
+
         try:
             if not utils.isunauthenticated(args.func):
                 self.cs.authenticate()
@@ -721,6 +760,28 @@ class OpenStackCinderShell(object):
                                "to the default API version: %s",
                                endpoint_api_version)
 
+        API_MAX_VERSION = api_versions.APIVersion(api_versions.MAX_VERSION)
+        if endpoint_api_version[0] == '3':
+            disc_client = client.Client(API_MAX_VERSION,
+                                        os_username,
+                                        os_password,
+                                        os_project_name,
+                                        os_auth_url,
+                                        **client_args)
+            self.cs, discovered_version = self._discover_client(
+                disc_client,
+                api_version,
+                args.os_endpoint_type,
+                args.service_type,
+                os_username,
+                os_password,
+                os_project_name,
+                os_auth_url,
+                client_args)
+
+            if discovered_version < api_version:
+                self.downgrade_warning(api_version, discovered_version)
+
         profile = osprofiler_profiler and options.profile
         if profile:
             osprofiler_profiler.init(options.profile)
@@ -734,6 +795,56 @@ class OpenStackCinderShell(object):
                 print("To display trace use next command:\n"
                       "osprofiler trace show --html %s " % trace_id)
 
+    def _discover_client(self,
+                         current_client,
+                         os_api_version,
+                         os_endpoint_type,
+                         os_service_type,
+                         os_username,
+                         os_password,
+                         os_project_name,
+                         os_auth_url,
+                         client_args):
+
+        if (os_api_version.get_major_version() in
+                api_versions.DEPRECATED_VERSIONS):
+            discovered_version = api_versions.DEPRECATED_VERSION
+            os_service_type = 'volume'
+        else:
+            discovered_version = api_versions.discover_version(
+                current_client,
+                os_api_version)
+
+        if not os_endpoint_type:
+            os_endpoint_type = DEFAULT_CINDER_ENDPOINT_TYPE
+
+        if not os_service_type:
+            os_service_type = self._discover_service_type(discovered_version)
+
+        API_MAX_VERSION = api_versions.APIVersion(api_versions.MAX_VERSION)
+
+        if (discovered_version != API_MAX_VERSION or
+                os_service_type != 'volume' or
+                os_endpoint_type != DEFAULT_CINDER_ENDPOINT_TYPE):
+            client_args['service_type'] = os_service_type
+            client_args['endpoint_type'] = os_endpoint_type
+
+            return (client.Client(discovered_version,
+                                  os_username,
+                                  os_password,
+                                  os_project_name,
+                                  os_auth_url,
+                                  **client_args),
+                    discovered_version)
+        else:
+            return current_client, discovered_version
+
+    def _discover_service_type(self, discovered_version):
+        SERVICE_TYPES = {'1': 'volume', '2': 'volumev2', '3': 'volumev3'}
+        major_version = discovered_version.get_major_version()
+        service_type = SERVICE_TYPES[major_version]
+        return service_type
+
     def _run_extension_hooks(self, hook_type, *args, **kwargs):
         """Runs hooks for all registered extensions."""
         for extension in self.extensions:
@@ -896,7 +1007,7 @@ class OpenStackHelpFormatter(argparse.HelpFormatter):
 
     def start_section(self, heading):
         # Title-case the headings
-        heading = '%s%s' % (heading[0].upper(), heading[1:])
+        heading = heading.title()
         super(OpenStackHelpFormatter, self).start_section(heading)
 
 
