@@ -15,7 +15,6 @@
 
 """Capabilities interface (v2 extension)"""
 
-
 from cinderclient import base
 
 
@@ -23,7 +22,7 @@ class Capabilities(base.Resource):
     NAME_ATTR = 'name'
 
     def __repr__(self):
-        return "<Capabilities: %s>" % self.name
+        return "<Capabilities: %s>" % self._info.get('namespace')
 
 
 class CapabilitiesManager(base.Manager):
